package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.ProjectProcessStageEnum;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.util.ProjectStateMachineUtil;
import com.hz.pm.api.expert.model.vo.ProjectReviewDetailVO;
import com.hz.pm.api.meeting.entity.domain.MeetingInnerProject;
import com.hz.pm.api.projectdeclared.entity.ProjectReview;
import com.hz.pm.api.projectdeclared.model.req.ProjectReviewApplyReq;
import com.hz.pm.api.projectdeclared.service.IProjectReviewService;
import com.hz.pm.api.projectlib.helper.ProjectHelper;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <p>
 * ProjectReviewManage
 * </p>
 *
 * @author WendyYang
 * @since 10:54 2024/4/17
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectReviewManage {

    private final IProjectReviewService projectReviewService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final IProjectService projectService;
    private final ProcessInstanceService processInstanceService;
    private final IProjectInstService projectInstService;
    private final ProjectStateMachineUtil projectStateMachineUtil;

    @Transactional(rollbackFor = Exception.class)
    public synchronized void projectReviewApply(ProjectReviewApplyReq req) {
        Project project = projectService.getById(req.getProjectId());
        if (!ProjectStatus.WITHOUT_PROJECT_REVIEW.eq(project.getStatus())
                && !ProjectStatus.PROJECT_REVIEW_FAILED.eq(project.getStatus())) {
            throw BizException.wrap("该项目不允许申请项目评审");
        }

        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        WflowModels model = processModelManage.getWflowModels(ProjectProcessStageEnum.PROJECT_REVIEW,
                user.getMhUnitId());

        if (Objects.isNull(model)) {
            throw BizException.wrap("找不到项目评审流程配置");
        }

        ProcessStartParamsVo processParam = new ProcessStartParamsVo();
        processParam.setUser(declaredProjectHelper.buildUser(user));
        processParam.setProcessUsers(Collections.emptyMap());
        processParam.setFormData(Collections.emptyMap());
        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Map<String, OrgInfoDTO> orgMap = declaredProjectHelper.buildOrgModelMap(user.getUserId(), project);
        String instanceId = processInstanceService.startProcessLs(model, processParam, orgMap);

        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstCode(instanceId);
        projectInst.setInstType(ProjectProcessStageEnum.PROJECT_REVIEW);
        projectInst.setProjectId(req.getProjectId());
        projectInstService.save(projectInst);
        projectStateMachineUtil.pass(project);
        projectService.updateById(project);
        ProjectReview projReview = projectReviewService.getOne(Wrappers
                .lambdaQuery(ProjectReview.class)
                .eq(ProjectReview::getProjectId, req.getProjectId())
                .orderByDesc(ProjectReview::getId)
                .last(BizConst.LIMIT_1));
        if (projReview == null) {
            projReview = new ProjectReview();
            projReview.setProjectCode(project.getProjectCode());
        }
        BeanUtil.copyProperties(req, projReview);
        projectReviewService.saveOrUpdate(projReview);
        log.info("项目评审提交成功 【{}】", instanceId);
    }

    public ProjectReviewDetailVO projectReviewDetail(Long projectId) {
        ProjectReview projReview = projectReviewService.getOne(Wrappers
                .lambdaQuery(ProjectReview.class)
                .eq(ProjectReview::getProjectId, projectId)
                .orderByDesc(ProjectReview::getId)
                .last(BizConst.LIMIT_1));
        return BeanUtil.copyProperties(projReview, ProjectReviewDetailVO.class);
    }

    /**
     * 项目列表
     *
     * @param req
     * @return
     */
    public PageVo<ProjectLibListItemVO> pageReviewProject(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(req);
        query.in(Project::getProcessStatus,
                ProjectStatus.WITHOUT_PROJECT_REVIEW.getCode(),
                ProjectStatus.ON_PROJECT_REVIEW.getCode(),
                ProjectStatus.PROJECT_REVIEW_FAILED.getCode());
        // 当非预审申报的时候 是自己单位 当是预审申报的时候 要主管单位
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        preQuery(query, user);
        query.eq(Project::getNewest, Boolean.TRUE);
        Page<Project> page = projectService.page(req.page(), query);
        long total;
        if ((total = page.getTotal()) == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(),
                w -> BeanUtil.copyProperties(w, ProjectLibListItemVO.class));
        return PageVo.of(records, total);
    }

    //当非预审申报的时候 是自己单位 当是预审申报的时候 要主管单位 并且他是单位管理员
    private void preQuery(LambdaQueryWrapper<Project> query, UserInfoDetails user) {
        //如果当前登录是单位管理员
        String mhUnitId = user.getMhUnitIdStr();
        if (user.getIsOrgAdmin()) {
            query.and(s1 -> s1.eq(Project::getSuperOrgCode, mhUnitId)
                    .or(q2 -> q2.eq(Project::getBuildOrgCode, mhUnitId)));
        } else {
            //否则 只能看到 非预审 并且
            query.and(q2 -> q2.eq(Project::getBuildOrgCode, mhUnitId));
        }
    }

}
